﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using Infragistics.Controls.Charts;

namespace CustomSeries_RangeAreaWithUnknowPlotting
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

           ObservableCollection<ChartData>  data = ChartData.GenerateData(20);

           data[5].High = null;
           data[9].High = null;
           data[11].High = null;
           data[11].Low = null;
           data[18].Low = null;

           DataContext = data;
        }
    }

    public class CustomRangeAreaSeries : HorizontalRangeCategorySeries
    {

        #region Base Overrides

        protected override void RenderSeriesOverride(bool animate)
        {
            base.RenderSeriesOverride(animate);

            if (this.Viewport.IsEmpty || this.RootCanvas == null ||
 this.XAxis == null || this.YAxis == null ||
 this.ItemsSource == null)
            {
                return;
            }

            this.RootCanvas.Children.Clear();

            List<PointCollection> high = new List<PointCollection>() { new PointCollection() };
            List<PointCollection> low = new List<PointCollection>() { new PointCollection() };

            double index = 0;
            int currentLineIndex = 0;

            //Getting Points based on the ItemsSource
            foreach (var item in ItemsSource)
            {
                //Getting X Value
                double xValue = 0;

                if (XAxis is CategoryXAxis)
                    xValue = index;

                if (XAxis is CategoryDateTimeXAxis)
                {
                    CategoryDateTimeXAxis xAxis = XAxis as CategoryDateTimeXAxis;
                    DateTime dateValue = (DateTime)item.GetType().GetProperty(xAxis.DateTimeMemberPath).GetValue(item, null);
                    xValue = dateValue.Ticks;
                }

                //Getting High Y Value
                double yHighValue = 0;
                object yHighValObj = item.GetType().GetProperty(HighMemberPath).GetValue(item, null);

                //Getting Low Y Value
                double yLowValue = 0;
                object yLowValObj = item.GetType().GetProperty(LowMemberPath).GetValue(item, null);



                if (yHighValObj != null && yHighValObj != null)
                {
                    yHighValue = Convert.ToDouble(yHighValObj);
                    yLowValue = Convert.ToDouble(yLowValObj);
                }
                else
                {
                    if (UnknownValuePlotting == Infragistics.Controls.Charts.UnknownValuePlotting.DontPlot)
                    {
                        if (high.LastOrDefault().Count > 0)
                        {
                            high.Add(new PointCollection());
                            low.Add(new PointCollection());
                            currentLineIndex++;
                        }
                        continue;
                    }
                    else
                    {
                        continue;
                    }
                }



                ScalerParams xParams = new ScalerParams(this.SeriesViewer.WindowRect, this.Viewport, XAxis.IsInverted);
                ScalerParams yParams = new ScalerParams(this.SeriesViewer.WindowRect, this.Viewport, YAxis.IsInverted);
                double x = this.XAxis.GetScaledValue(xValue, xParams);

                double yH = this.YAxis.GetScaledValue(yHighValue, yParams);
                double yL = this.YAxis.GetScaledValue(yLowValue, yParams);

                high[currentLineIndex].Add(new Point(x, yH));
                low[currentLineIndex].Add(new Point(x, yL));

                index++;

            }

            Path path = new Path();
            path.Stroke = this.ActualBrush;
            path.Fill = this.ActualBrush;
            path.StrokeThickness = this.Thickness;

            PathGeometry areaGeometry = new PathGeometry();
            areaGeometry.Figures = new PathFigureCollection();

            for(int p = 0; p < high.Count; p++)
            {
                PathFigure areaFigure = new PathFigure();
                areaFigure.IsClosed = true;
                areaFigure.StartPoint = high[p][0];


                PointCollection finalPoints = new PointCollection();

                for (int i = 1; i < high[p].Count; i++)
                {
                    LineSegment segment = new LineSegment(high[p][i], true);
                    areaFigure.Segments.Add(segment);
                }

                for (int i = low[p].Count - 1; i >= 0; i--)
                {
                    LineSegment segment = new LineSegment(low[p][i], true);
                    areaFigure.Segments.Add(segment);
                }

                areaGeometry.Figures.Add(areaFigure);
            }



            path.Data = areaGeometry;


            this.RootCanvas.Children.Add(path);
        }

        #endregion

        #region Properties

        public UnknownValuePlotting UnknownValuePlotting
        {
            get { return (UnknownValuePlotting)GetValue(UnknownValuePlottingProperty); }
            set { SetValue(UnknownValuePlottingProperty, value); }
        }

        // Using a DependencyProperty as the backing store for UnknownValuePlotting.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty UnknownValuePlottingProperty =
            DependencyProperty.Register(
                "UnknownValuePlotting",
                typeof(UnknownValuePlotting),
                typeof(CustomRangeAreaSeries),
                new PropertyMetadata(UnknownValuePlotting.DontPlot, 
                    (o,e) =>
                    {
                        (o as CustomRangeAreaSeries).RenderSeriesOverride(false);
                    }));



        #endregion

    }


    public class ChartData
    {
        public DateTime Date { get; set; }
        public int Index { get; set; }
        public double? High { get; set; }
        public double? Low { get; set; }

        public static ObservableCollection<ChartData> GenerateData(int count)
        {
            ObservableCollection<ChartData> data = new ObservableCollection<ChartData>();

            Random r = new Random();

            ChartData data0 = new ChartData
            {
                Date = DateTime.Now,
                Index = 0,
                High = r.Next(50, 60),
                Low = r.Next(0, 10),
            };

            data.Add(data0);

            for (int i = 1; i < count; i++)
            {
                ChartData dat = new ChartData
                {
                    Date = DateTime.Now.AddDays(i),
                    Index = i,
                };

                if (r.NextDouble() > .5)
                {
                    dat.High = data[i - 1].High + r.NextDouble();
                    dat.Low = data[i - 1].Low + r.NextDouble();
                }
                else
                {
                    dat.High = data[i - 1].High - r.NextDouble();
                    dat.Low = data[i - 1].Low - r.NextDouble();
                }

                data.Add(dat);
            }

            return data;
        }
    }


}
